--[[TABLE PARData
Fields = {
TABLE Options
Fields = {
}
ARRAY AbilityID (unnamed of course)
Fields = {
	1-999ish
	TotalHits  calculated on every event
	TotalMisses  calculated on every event
	
}
TotalHits
TotalMisses
Average]]--

-- set name and namespace
addonname, PAR = ...

-- shortcut
local CPB = C_PetBattles

-- lazy, easy condition evaluation functions
local function Any(v, t) for i = 1, #t do if (v == t[i]) then return true end end return false end
local function NotAny(v, t) for i = 1, #t do if (v == t[i]) then return false end end return true end

-- Debugging (remove hyphens from the following line to activate)
--local DEBUGGING = true

-- create event handling frame and register event(s)
local PAREventFrame = CreateFrame("Frame", "PAREventFrame")
PAREventFrame:RegisterEvent("ADDON_LOADED")
PAREventFrame:RegisterEvent("CHAT_MSG_PET_BATTLE_COMBAT_LOG")

function ResetPARData()
	PARData = {}
	PARData.Miss = 0
	PARData.Hit = 0
	PARData.Total = 0
end

local function PAREventFrame_OnEvent(self, event, message, ...)
	if (event == "ADDON_LOADED") and (message == addonname) then
		
		-- create initial SavedVariables if not found
		if (not PARData) then
			PARData = {}
			PARData.Miss = 0
			PARData.Hit = 0
			PARData.Total = 0
		end
		
		-- unregister event now that addon has loaded
		PAREventFrame:UnregisterEvent("ADDON_LOADED")
	elseif (event == "CHAT_MSG_PET_BATTLE_COMBAT_LOG") then
	
		-- RETURN if not a miss, buff/debuff application, or damaging attack
		if (not strmatch(message, " applied ")) and (not strmatch(message, " missed ")) and (not strmatch(message, " dealt ")) then
			if DEBUGGING then
				print("Not a miss, application, or damage.")
			end
			return
		end
		
		-- retrieve abilityID from message (first abilityID is only relevant one, so ignoring possiblity of others)
		local startpos = strfind(message, "battlePetAbil")
		local _, endpos = strfind(message, "|h|r")
		local _, abilityID, _ = strsplit(":", strsub(message, startpos, endpos))
		abilityID = tonumber(abilityID)
		
		-- RETURN if abilityID doesn't match any AcceptableAbilities
		--[[if (not abilityID) or NotAny(abilityID, PAR.AcceptableAbilities) then
			if DEBUGGING and abilityID then
				print("Ability with ID " .. abilityID .. " is not on the list of acceptable abilities.")
			end
			return
		end]]--
		
		-- Find out which player is doing the attack
		-- " enemy " means you did the attack
		-- " your " means the enemy did the attack
		local petOwner, enemyOwner
		if strmatch(message, " enemy ") then
			petOwner = 1
			enemyOwner = 2
		elseif strmatch(message, " your ") then
			petOwner = 2
			enemyOwner = 1
		else
			if DEBUGGING then
				print("Couldn't figure out petOwner/enemyOwner.")
			end
			return
		end
		
		-- find active pet
		local petIndex = CPB.GetActivePet(petOwner)
		local enemyIndex = CPB.GetActivePet(enemyOwner)
		
		-- RETURN if pets are not the same level
		local petLevel = CPB.GetLevel(petOwner, petIndex)
		local enemyLevel = CPB.GetLevel(enemyOwner, enemyIndex)
		if (petLevel ~= enemyLevel) then
			if DEBUGGING then
				print("Active pets are not the same level.")
			end
			return
		end
		
		-- RETURN if opposite pet has an aura from MissAuras
		local numAuras = CPB.GetNumAuras(enemyOwner, enemyIndex)
		if (numAuras) and (numAuras > 0) then
			for auraIndex = 1, numAuras do
				local currentAura = CPB.GetAuraInfo(enemyOwner, enemyIndex, auraIndex)
				if Any(currentAura, PAR.MissAuras) then
					if DEBUGGING then
						print("There is an Aura on the enemy pet that causes misses.")
					end
					return
				end
			end	
		end
		
		-- calculate AccuracyModifier using StateValues (alternate approach: use AccuracyAuras array)
		local AccuracyModifier
		if (CPB.GetPetType(petOwner, petIndex) == 7) then --Elementals aren't affected by weathers.
			AccuracyModifier = CPB.GetStateValue(petOwner, petIndex, 41) + CPB.GetStateValue(petOwner, 0, 41)
		else
			AccuracyModifier = CPB.GetStateValue(petOwner, petIndex, 41) + CPB.GetStateValue(petOwner, 0, 41) + CPB.GetStateValue(0, 0, 41)
		end
		
		-- Obtain raw accuracy value for ability by looping through possible turns/effects
		-- since I'm picking simple abilities only (see AcceptableAbilities), the first result will be the relevant result
		-- so break out of the function early when we find a result
		local RawAccuracy
		for turnIndex = 1, 10 do
			for effectIndex = 1, 10 do
				RawAccuracy = CPB.GetAbilityEffectInfo(abilityID, turnIndex, effectIndex, "accuracy")
				if RawAccuracy then break end
			end
			if RawAccuracy then break end
		end
		
		-- RETURN if we don't find any accuracy value
		if not RawAccuracy then
			if DEBUGGING then
				print("Error, found no accuracy result for ability with ID " .. abilityID)
			end
			return
		end
		
		-- combine Accuracy
		local totalAccuracy = RawAccuracy + AccuracyModifier
		
		-- create table structure if empty previously
		if (not PARData[totalAccuracy]) then
			PARData[totalAccuracy] = {}
			PARData[totalAccuracy].Miss = 0
			PARData[totalAccuracy].Hit = 0
			PARData[totalAccuracy].Total = 0
		end
		if (not PARData[totalAccuracy][abilityID]) then
			PARData[totalAccuracy][abilityID] = {}
			PARData[totalAccuracy][abilityID].Miss = 0
			PARData[totalAccuracy][abilityID].Hit = 0
			PARData[totalAccuracy][abilityID].Total = 0
		end
		
		
		-- record data, separate into miss and hit (dealt/applied)
		-- table structure is PARData[totalAccuracy][abilityID].Miss (and Hit and Total)
		-- then follow up by looking at PARData[totalAccuracy].Miss
		-- and lastly PARData.Miss
		if (strmatch(message, " applied ")) or (strmatch(message, " dealt ")) then
			
			-- add a "Hit" result
			PARData[totalAccuracy][abilityID].Hit = PARData[totalAccuracy][abilityID].Hit + 1
			PARData[totalAccuracy].Hit = PARData[totalAccuracy].Hit + 1
			PARData.Hit = PARData.Hit + 1
			
			-- increment total number of recorded attacks
			PARData[totalAccuracy][abilityID].Total = PARData[totalAccuracy][abilityID].Total + 1
			PARData[totalAccuracy].Total = PARData[totalAccuracy].Total + 1
			PARData.Total = PARData.Total + 1
		elseif (strmatch(message, " missed ")) then
			
			-- add a "Miss" result
			PARData[totalAccuracy][abilityID].Miss = PARData[totalAccuracy][abilityID].Miss + 1
			PARData[totalAccuracy].Miss = PARData[totalAccuracy].Miss + 1
			PARData.Miss = PARData.Miss + 1
			
			-- increment total number of recorded attacks
			PARData[totalAccuracy][abilityID].Total = PARData[totalAccuracy][abilityID].Total + 1
			PARData[totalAccuracy].Total = PARData[totalAccuracy].Total + 1
			PARData.Total = PARData.Total + 1
		else
			if DEBUGGING then
				print("Wait, how'd this even happen?")
			end
			return
		end
	end
end

-- set our event handler function
PAREventFrame:SetScript("OnEvent", PAREventFrame_OnEvent)

-- slash command handling
SLASH_PETACCURACYRECORDER1 = "/PAR"
SLASH_PETACCURACYRECORDER2 = "/petaccuracy"
SlashCmdList["PETACCURACYRECORDER"] = function(message)
	
	-- first find ExpectedOverallAccuracy
	local ExpectedOverallAccuracy = 0
	
	-- use totalAccuracy * prevalence, where prevalence is calculated as (PARData[totalAccuracy].Total / PARData.Total)
	for k, v in next, PARData do
		if (k) and (type(v) == "table") then
			ExpectedOverallAccuracy = ExpectedOverallAccuracy + ((k / 100) * (PARData[k].Total / PARData.Total))
		end
	end
	
	-- next find TrueOverallAccuracy, calculated as (PARData.Hit / PARData.Total)
	local TrueOverallAccuracy = PARData.Hit / PARData.Total
	
	-- save results to data table
	PARData.Expected = ExpectedOverallAccuracy
	PARData.True = TrueOverallAccuracy
	
	-- show results to user
	print(format("Expected Accuracy: %.2f%%", PARData.Expected * 100))
	print(format("Actual Accuracy: %.2f%%", PARData.True * 100))
	
	-- do statistical analysis (obtain std deviation and perform t-test) in future version
	--[[for k, v in next, PARData do
		if (k) and (type(v) == "table") then
			ExpectedOverallAccuracy = ExpectedOverallAccuracy + ((k / 100) * (PARData[k].Total / PARData.Total))
		end
	end]]--
end